@@ -8,12 +8,13 @@ from functools import lru_cache
 from pathlib import Path
 from typing import TYPE_CHECKING, ClassVar, Iterable, Optional, Sequence, Tuple
 
+from rich.console import RenderableType
 from rich.style import Style
 from rich.text import Text
 from typing_extensions import Literal
 
 from textual._text_area_theme import TextAreaTheme
-from textual._tree_sitter import TREE_SITTER
+from textual._tree_sitter import BUILTIN_LANGUAGES, TREE_SITTER
 from textual.color import Color
 from textual.document._document import (
     Document,
@@ -26,7 +27,6 @@ from textual.document._document import (
 from textual.document._document_navigator import DocumentNavigator
 from textual.document._edit import Edit
 from textual.document._history import EditHistory
-from textual.document._languages import BUILTIN_LANGUAGES
 from textual.document._syntax_aware_document import (
     SyntaxAwareDocument,
     SyntaxAwareDocumentError,
@@ -35,7 +35,7 @@ from textual.document._wrapped_document import WrappedDocument
 from textual.expand_tabs import expand_tabs_inline, expand_text_tabs_from_widths
 
 if TYPE_CHECKING:
-    from tree_sitter import Language
+    from tree_sitter import Language, Query
 
 from textual import events, log
 from textual._cells import cell_len, cell_width_to_column_index
@@ -90,15 +90,19 @@ class TextArea(ScrollView):
 TextArea {
     width: 1fr;
     height: 1fr;
-    border: tall $background;
+    border: tall $border-blurred;
     padding: 0 1;
-
+    color: $foreground;
+    background: $surface;
+    & .text-area--cursor {
+        text-style: $input-cursor-text-style;
+    }
     & .text-area--gutter {
-        color: $text 40%;
+        color: $foreground 40%;
     }
 
     & .text-area--cursor-gutter {
-        color: $text 60%;
+        color: $foreground 60%;
         background: $boost;
         text-style: bold;
     }
@@ -108,7 +112,7 @@ TextArea {
     }
 
     & .text-area--selection {
-        background: $accent-lighten-1 40%;
+        background: $input-selection-background;
     }
 
     & .text-area--matching-bracket {
@@ -116,13 +120,20 @@ TextArea {
     }
 
     &:focus {
-        border: tall $accent;
+        border: tall $border;
+    }
+
+    &:ansi {
+        & .text-area--selection {
+            background: transparent;
+            text-style: reverse;
+        }
     }
 
     &:dark {
         .text-area--cursor {
-           color: $text 90%;
-            background: $foreground 90%;
+            color: $input-cursor-foreground;
+            background: $input-cursor-background;
         }
         &.-read-only .text-area--cursor {
             background: $warning-darken-1;
@@ -166,63 +177,71 @@ TextArea {
 
     BINDINGS = [
         # Cursor movement
-        Binding("up", "cursor_up", "cursor up", show=False),
-        Binding("down", "cursor_down", "cursor down", show=False),
-        Binding("left", "cursor_left", "cursor left", show=False),
-        Binding("right", "cursor_right", "cursor right", show=False),
-        Binding("ctrl+left", "cursor_word_left", "cursor word left", show=False),
-        Binding("ctrl+right", "cursor_word_right", "cursor word right", show=False),
-        Binding("home,ctrl+a", "cursor_line_start", "cursor line start", show=False),
-        Binding("end,ctrl+e", "cursor_line_end", "cursor line end", show=False),
-        Binding("pageup", "cursor_page_up", "cursor page up", show=False),
-        Binding("pagedown", "cursor_page_down", "cursor page down", show=False),
+        Binding("up", "cursor_up", "Cursor up", show=False),
+        Binding("down", "cursor_down", "Cursor down", show=False),
+        Binding("left", "cursor_left", "Cursor left", show=False),
+        Binding("right", "cursor_right", "Cursor right", show=False),
+        Binding("ctrl+left", "cursor_word_left", "Cursor word left", show=False),
+        Binding("ctrl+right", "cursor_word_right", "Cursor word right", show=False),
+        Binding("home,ctrl+a", "cursor_line_start", "Cursor line start", show=False),
+        Binding("end,ctrl+e", "cursor_line_end", "Cursor line end", show=False),
+        Binding("pageup", "cursor_page_up", "Cursor page up", show=False),
+        Binding("pagedown", "cursor_page_down", "Cursor page down", show=False),
         # Making selections (generally holding the shift key and moving cursor)
         Binding(
             "ctrl+shift+left",
             "cursor_word_left(True)",
-            "cursor left word select",
+            "Cursor left word select",
             show=False,
         ),
         Binding(
             "ctrl+shift+right",
             "cursor_word_right(True)",
-            "cursor right word select",
+            "Cursor right word select",
             show=False,
         ),
         Binding(
             "shift+home",
             "cursor_line_start(True)",
-            "cursor line start select",
+            "Cursor line start select",
             show=False,
         ),
         Binding(
-            "shift+end", "cursor_line_end(True)", "cursor line end select", show=False
+            "shift+end", "cursor_line_end(True)", "Cursor line end select", show=False
         ),
-        Binding("shift+up", "cursor_up(True)", "cursor up select", show=False),
-        Binding("shift+down", "cursor_down(True)", "cursor down select", show=False),
-        Binding("shift+left", "cursor_left(True)", "cursor left select", show=False),
-        Binding("shift+right", "cursor_right(True)", "cursor right select", show=False),
+        Binding("shift+up", "cursor_up(True)", "Cursor up select", show=False),
+        Binding("shift+down", "cursor_down(True)", "Cursor down select", show=False),
+        Binding("shift+left", "cursor_left(True)", "Cursor left select", show=False),
+        Binding("shift+right", "cursor_right(True)", "Cursor right select", show=False),
         # Shortcut ways of making selections
         # Binding("f5", "select_word", "select word", show=False),
-        Binding("f6", "select_line", "select line", show=False),
-        Binding("f7", "select_all", "select all", show=False),
+        Binding("f6", "select_line", "Select line", show=False),
+        Binding("f7", "select_all", "Select all", show=False),
         # Deletion
-        Binding("backspace", "delete_left", "delete left", show=False),
+        Binding("backspace", "delete_left", "Delete character left", show=False),
         Binding(
-            "ctrl+w", "delete_word_left", "delete left to start of word", show=False
+            "ctrl+w", "delete_word_left", "Delete left to start of word", show=False
         ),
-        Binding("delete,ctrl+d", "delete_right", "delete right", show=False),
+        Binding("delete,ctrl+d", "delete_right", "Delete character right", show=False),
         Binding(
-            "ctrl+f", "delete_word_right", "delete right to start of word", show=False
+            "ctrl+f", "delete_word_right", "Delete right to start of word", show=False
         ),
-        Binding("ctrl+x", "delete_line", "delete line", show=False),
+        Binding("ctrl+x", "cut", "Cut", show=False),
+        Binding("ctrl+c", "copy", "Copy", show=False),
+        Binding("ctrl+v", "paste", "Paste", show=False),
         Binding(
-            "ctrl+u", "delete_to_start_of_line", "delete to line start", show=False
+            "ctrl+u", "delete_to_start_of_line", "Delete to line start", show=False
         ),
         Binding(
             "ctrl+k",
             "delete_to_end_of_line_or_delete_line",
-            "delete to line end",
+            "Delete to line end",
+            show=False,
+        ),
+        Binding(
+            "ctrl+shift+k",
+            "delete_line",
+            "Delete line",
             show=False,
         ),
         Binding("ctrl+z", "undo", "Undo", show=False),
@@ -253,13 +272,16 @@ TextArea {
     | ctrl+w                 | Delete from cursor to start of the word.     |
     | delete,ctrl+d          | Delete character to the right of cursor.     |
     | ctrl+f                 | Delete from cursor to end of the word.       |
-    | ctrl+x                 | Delete the current line.                     |
+    | ctrl+shift+k           | Delete the current line.                     |
     | ctrl+u                 | Delete from cursor to the start of the line. |
     | ctrl+k                 | Delete from cursor to the end of the line.   |
     | f6                     | Select the current line.                     |
     | f7                     | Select all text in the document.             |
     | ctrl+z                 | Undo.                                        |
     | ctrl+y                 | Redo.                                        |
+    | ctrl+x                 | Cut selection or line if no selection.       |
+    | ctrl+c                 | Copy selection to clipboard.                 |
+    | ctrl+v                 | Paste from clipboard.                        |
     """
 
     language: Reactive[str | None] = reactive(None, always_update=True, init=False)
@@ -300,6 +322,9 @@ TextArea {
 
     Changing this value will immediately re-render the `TextArea`."""
 
+    line_number_start: Reactive[int] = reactive(1, init=False)
+    """The line number the first line should be."""
+
     indent_width: Reactive[int] = reactive(4, init=False)
     """The width of tabs or the multiple of spaces to align to on pressing the `tab` key.
 
@@ -369,11 +394,13 @@ TextArea {
         tab_behavior: Literal["focus", "indent"] = "focus",
         read_only: bool = False,
         show_line_numbers: bool = False,
+        line_number_start: int = 1,
         max_checkpoints: int = 50,
         name: str | None = None,
         id: str | None = None,
         classes: str | None = None,
         disabled: bool = False,
+        tooltip: RenderableType | None = None,
     ) -> None:
         """Construct a new `TextArea`.
 
@@ -385,17 +412,26 @@ TextArea {
             tab_behavior: If 'focus', pressing tab will switch focus. If 'indent', pressing tab will insert a tab.
             read_only: Enable read-only mode. This prevents edits using the keyboard.
             show_line_numbers: Show line numbers on the left edge.
+            line_number_start: What line number to start on.
             max_checkpoints: The maximum number of undo history checkpoints to retain.
             name: The name of the `TextArea` widget.
             id: The ID of the widget, used to refer to it from Textual CSS.
             classes: One or more Textual CSS compatible class names separated by spaces.
             disabled: True if the widget is disabled.
+            tooltip: Optional tooltip.
         """
         super().__init__(name=name, id=id, classes=classes, disabled=disabled)
 
         self._languages: dict[str, TextAreaLanguage] = {}
         """Maps language names to TextAreaLanguage."""
 
+        for language_name, language_object in BUILTIN_LANGUAGES.items():
+            self._languages[language_name] = TextAreaLanguage(
+                language_name,
+                language_object,
+                self._get_builtin_highlight_query(language_name),
+            )
+
         self._themes: dict[str, TextAreaTheme] = {}
         """Maps theme names to TextAreaTheme."""
 
@@ -452,11 +488,12 @@ TextArea {
         self.set_reactive(TextArea.soft_wrap, soft_wrap)
         self.set_reactive(TextArea.read_only, read_only)
         self.set_reactive(TextArea.show_line_numbers, show_line_numbers)
+        self.set_reactive(TextArea.line_number_start, line_number_start)
 
         self.tab_behavior = tab_behavior
 
-        # When `app.dark` is toggled, reset the theme (since it caches values).
-        self.watch(self.app, "dark", self._app_dark_toggled, init=False)
+        if tooltip is not None:
+            self.tooltip = tooltip
 
     @classmethod
     def code_editor(
@@ -469,11 +506,13 @@ TextArea {
         tab_behavior: Literal["focus", "indent"] = "indent",
         read_only: bool = False,
         show_line_numbers: bool = True,
+        line_number_start: int = 1,
         max_checkpoints: int = 50,
         name: str | None = None,
         id: str | None = None,
         classes: str | None = None,
         disabled: bool = False,
+        tooltip: RenderableType | None = None,
     ) -> TextArea:
         """Construct a new `TextArea` with sensible defaults for editing code.
 
@@ -487,10 +526,12 @@ TextArea {
             soft_wrap: Enable soft wrapping.
             tab_behavior: If 'focus', pressing tab will switch focus. If 'indent', pressing tab will insert a tab.
             show_line_numbers: Show line numbers on the left edge.
+            line_number_start: What line number to start on.
             name: The name of the `TextArea` widget.
             id: The ID of the widget, used to refer to it from Textual CSS.
             classes: One or more Textual CSS compatible class names separated by spaces.
             disabled: True if the widget is disabled.
+            tooltip: Optional tooltip
         """
         return cls(
             text,
@@ -500,11 +541,13 @@ TextArea {
             tab_behavior=tab_behavior,
             read_only=read_only,
             show_line_numbers=show_line_numbers,
+            line_number_start=line_number_start,
             max_checkpoints=max_checkpoints,
             name=name,
             id=id,
             classes=classes,
             disabled=disabled,
+            tooltip=tooltip,
         )
 
     @staticmethod
@@ -522,12 +565,33 @@ TextArea {
                 Path(_HIGHLIGHTS_PATH.resolve()) / f"{language_name}.scm"
             )
             highlight_query = highlight_query_path.read_text()
-        except OSError as e:
-            log.warning(f"Unable to load highlight query. {e}")
+        except OSError as error:
+            log.warning(f"Unable to load highlight query. {error}")
             highlight_query = ""
 
         return highlight_query
 
+    def check_consume_key(self, key: str, character: str | None = None) -> bool:
+        """Check if the widget may consume the given key.
+
+        As a textarea we are expecting to capture printable keys.
+
+        Args:
+            key: A key identifier.
+            character: A character associated with the key, or `None` if there isn't one.
+
+        Returns:
+            `True` if the widget may capture the key in it's `Key` message, or `False` if it won't.
+        """
+        if self.read_only:
+            # In read only mode we don't consume any key events
+            return False
+        if self.tab_behavior == "indent" and key == "tab":
+            # If tab_behavior is indent, then we consume the tab
+            return True
+        # Otherwise we capture all printable keys
+        return character is not None and character.isprintable()
+
     def _build_highlight_map(self) -> None:
         """Query the tree for ranges to highlights, and update the internal highlights mapping."""
         highlights = self._highlights
@@ -536,26 +600,28 @@ TextArea {
             return
 
         captures = self.document.query_syntax_tree(self._highlight_query)
-        for capture in captures:
-            node, highlight_name = capture
-            node_start_row, node_start_column = node.start_point
-            node_end_row, node_end_column = node.end_point
+        for highlight_name, nodes in captures.items():
+            for node in nodes:
+                node_start_row, node_start_column = node.start_point
+                node_end_row, node_end_column = node.end_point
 
-            if node_start_row == node_end_row:
-                highlight = (node_start_column, node_end_column, highlight_name)
-                highlights[node_start_row].append(highlight)
-            else:
-                # Add the first line of the node range
-                highlights[node_start_row].append(
-                    (node_start_column, None, highlight_name)
-                )
+                if node_start_row == node_end_row:
+                    highlight = (node_start_column, node_end_column, highlight_name)
+                    highlights[node_start_row].append(highlight)
+                else:
+                    # Add the first line of the node range
+                    highlights[node_start_row].append(
+                        (node_start_column, None, highlight_name)
+                    )
 
-                # Add the middle lines - entire row of this node is highlighted
-                for node_row in range(node_start_row + 1, node_end_row):
-                    highlights[node_row].append((0, None, highlight_name))
+                    # Add the middle lines - entire row of this node is highlighted
+                    for node_row in range(node_start_row + 1, node_end_row):
+                        highlights[node_row].append((0, None, highlight_name))
 
-                # Add the last line of the node range
-                highlights[node_end_row].append((0, node_end_column, highlight_name))
+                    # Add the last line of the node range
+                    highlights[node_end_row].append(
+                        (0, node_end_column, highlight_name)
+                    )
 
     def _watch_has_focus(self, focus: bool) -> None:
         self._cursor_visible = focus
@@ -571,6 +637,10 @@ TextArea {
     ) -> None:
         """When the cursor moves, scroll it into view."""
         # Find the visual offset of the cursor in the document
+
+        if not self.is_mounted:
+            return
+
         cursor_location = selection.end
 
         self.scroll_cursor_visible()
@@ -627,6 +697,7 @@ TextArea {
         match_location = None
         bracket_stack: list[str] = []
         if bracket in _OPENING_BRACKETS:
+            # Search forwards for a closing bracket
             for candidate, candidate_location in self._yield_character_locations(
                 search_from
             ):
@@ -642,6 +713,7 @@ TextArea {
                             match_location = candidate_location
                             break
         elif bracket in _CLOSING_BRACKETS:
+            # Search backwards for an opening bracket
             for (
                 candidate,
                 candidate_location,
@@ -668,6 +740,9 @@ TextArea {
 
     def _watch_language(self, language: str | None) -> None:
         """When the language is updated, update the type of document."""
+        if not TREE_SITTER:
+            return
+
         if language is not None and language not in self.available_languages:
             raise LanguageDoesNotExist(
                 f"{language!r} is not a builtin language, or it has not been registered. "
@@ -682,6 +757,11 @@ TextArea {
         self._rewrap_and_refresh_virtual_size()
         self.scroll_cursor_visible()
 
+    def _watch_line_number_start(self) -> None:
+        """The line number gutter max size might change and contributes to virtual size, so recalculate."""
+        self._rewrap_and_refresh_virtual_size()
+        self.scroll_cursor_visible()
+
     def _watch_indent_width(self) -> None:
         """Changing width of tabs will change the document display width."""
         self._rewrap_and_refresh_virtual_size()
@@ -697,7 +777,7 @@ TextArea {
         if padding is applied, the colors match."""
         self._set_theme(theme)
 
-    def _app_dark_toggled(self) -> None:
+    def _app_theme_changed(self) -> None:
         self._set_theme(self._theme.name)
 
     def _set_theme(self, theme: str) -> None:
@@ -772,7 +852,8 @@ TextArea {
 
     def register_language(
         self,
-        language: "str | Language",
+        name: str,
+        language: "Language",
         highlight_query: str,
     ) -> None:
         """Register a language and corresponding highlight query.
@@ -787,34 +868,29 @@ TextArea {
         Registering a language only registers it to this instance of `TextArea`.
 
         Args:
-            language: A string referring to a builtin language or a tree-sitter `Language` object.
+            name: The name of the language.
+            language: A tree-sitter `Language` object.
             highlight_query: The highlight query to use for syntax highlighting this language.
         """
-
-        # If tree-sitter is unavailable, do nothing.
         if not TREE_SITTER:
             return
+        self._languages[name] = TextAreaLanguage(name, language, highlight_query)
 
-        from tree_sitter_languages import get_language
+    def update_highlight_query(self, name: str, highlight_query: str) -> None:
+        """Update the highlight query for an already registered language.
 
-        if isinstance(language, str):
-            language_name = language
-            language = get_language(language_name)
-        else:
-            language_name = language.name
-
-        # Update the custom languages. When changing the document,
-        # we should first look in here for a language specification.
-        # If nothing is found, then we can go to the builtin languages.
-        self._languages[language_name] = TextAreaLanguage(
-            name=language_name,
-            language=language,
-            highlight_query=highlight_query,
-        )
-        # If we updated the currently set language, rebuild the highlights
-        # using the newly updated highlights query.
-        if language_name == self.language:
-            self._set_document(self.text, language_name)
+        Args:
+            name: The name of the language.
+            highlight_query: The highlight query to use for syntax highlighting this language.
+        """
+        if name not in self._languages:
+            raise LanguageDoesNotExist(
+                f"{name!r} is not a registered language.\n"
+                f"To register a language, call `TextArea.register_language`."
+            )
+        self._languages[name].highlight_query = highlight_query
+        if name == self.language:
+            self._set_document(self.text, name)
 
     def _set_document(self, text: str, language: str | None) -> None:
         """Construct and return an appropriate document.
@@ -1133,7 +1209,9 @@ TextArea {
                 gutter_style = theme.gutter_style
 
             gutter_width_no_margin = gutter_width - 2
-            gutter_content = str(line_index + 1) if section_offset == 0 else ""
+            gutter_content = (
+                str(line_index + self.line_number_start) if section_offset == 0 else ""
+            )
             gutter = Text(
                 f"{gutter_content:>{gutter_width_no_margin}}  ",
                 style=gutter_style or "",
@@ -1187,9 +1265,7 @@ TextArea {
 
         # Crop the line to show only the visible part (some may be scrolled out of view)
         if not self.soft_wrap:
-            text_strip = text_strip.crop(
-                scroll_x, scroll_x + virtual_width - gutter_width
-            )
+            text_strip = text_strip.crop(scroll_x, scroll_x + virtual_width)
 
         # Stylize the line the cursor is currently on.
         if cursor_row == line_index:
@@ -1228,6 +1304,11 @@ TextArea {
         start, end = self.selection
         return self.get_text_range(start, end)
 
+    @property
+    def matching_bracket_location(self) -> Location | None:
+        """The location of the matching bracket, if there is one."""
+        return self._matching_bracket_location
+
     def get_text_range(self, start: Location, end: Location) -> str:
         """Get the text between a start and end location.
 
@@ -1458,13 +1539,17 @@ TextArea {
         # The longest number in the gutter plus two extra characters: `│ `.
         gutter_margin = 2
         gutter_width = (
-            len(str(self.document.line_count)) + gutter_margin
+            len(str(self.document.line_count - 1 + self.line_number_start))
+            + gutter_margin
             if self.show_line_numbers
             else 0
         )
         return gutter_width
 
     def _on_mount(self, event: events.Mount) -> None:
+        # When `app.theme` reactive is changed, reset the theme to clear cached styles.
+        self.watch(self.app, "theme", self._app_theme_changed, init=False)
+
         self.blink_timer = self.set_interval(
             0.5,
             self._toggle_cursor_blink_visible,
@@ -1513,17 +1598,18 @@ TextArea {
 
     def _end_mouse_selection(self) -> None:
         """Finalize the selection that has been made using the mouse."""
-        self._selecting = False
-        self.release_mouse()
-        self.record_cursor_width()
-        self._restart_blink()
+        if self._selecting:
+            self._selecting = False
+            self.release_mouse()
+            self.record_cursor_width()
+            self._restart_blink()
 
     async def _on_mouse_up(self, event: events.MouseUp) -> None:
         """Finalize the selection that has been made using the mouse."""
         self._end_mouse_selection()
 
     async def _on_hide(self, event: events.Hide) -> None:
-        """Finalize the selection that has been made using the mouse when thew widget is hidden."""
+        """Finalize the selection that has been made using the mouse when the widget is hidden."""
         self._end_mouse_selection()
 
     async def _on_paste(self, event: events.Paste) -> None:
@@ -2106,6 +2192,10 @@ TextArea {
 
     def action_delete_line(self) -> None:
         """Deletes the lines which intersect with the selection."""
+        self._delete_cursor_line()
+
+    def _delete_cursor_line(self) -> EditResult | None:
+        """Deletes the line (including the line terminator) that the cursor is on."""
         start, end = self.selection
         start, end = sorted((start, end))
         start_row, _start_column = start
@@ -2122,6 +2212,34 @@ TextArea {
         deletion = self._delete_via_keyboard(from_location, to_location)
         if deletion is not None:
             self.move_cursor_relative(columns=end_column, record_width=False)
+        return deletion
+
+    def action_cut(self) -> None:
+        """Cut text (remove and copy to clipboard)."""
+        if self.read_only:
+            return
+        start, end = self.selection
+        if start == end:
+            edit_result = self._delete_cursor_line()
+        else:
+            edit_result = self._delete_via_keyboard(start, end)
+
+        if edit_result is not None:
+            self.app.copy_to_clipboard(edit_result.replaced_text)
+
+    def action_copy(self) -> None:
+        """Copy selection to clipboard."""
+        selected_text = self.selected_text
+        if selected_text:
+            self.app.copy_to_clipboard(selected_text)
+
+    def action_paste(self) -> None:
+        """Paste from local clipboard."""
+        if self.read_only:
+            return
+        clipboard = self.app.clipboard
+        if result := self._replace_via_keyboard(clipboard, *self.selection):
+            self.move_cursor(result.end_location)
 
     def action_delete_to_start_of_line(self) -> None:
         """Deletes from the cursor location to the start of the line."""
